# Generated by Django 5.0.4 on 2024-04-17 01:54

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='BgTask',
            fields=[
                ('task_id', models.CharField(max_length=127, primary_key=True, serialize=False, verbose_name='Task ID')),
                ('fn', models.CharField(max_length=1024, verbose_name='Function Name')),
                ('args', models.BinaryField(blank=True, max_length=1024, null=True)),
                ('kwargs', models.BinaryField(blank=True, max_length=1024, null=True)),
                ('err_msg', models.CharField(blank=True, max_length=1024, null=True)),
                ('state', models.CharField(choices=[('Wait', 'Wait'), ('Ready', 'Ready'), ('Running', 'Running'), ('Finish', 'Finish')], max_length=20)),
                ('result', models.CharField(blank=True, choices=[('Success', 'Success'), ('Fail', 'Fail')], max_length=20, null=True)),
                ('return_value', models.BinaryField(blank=True, max_length=1024, null=True)),
                ('start_time', models.DateTimeField(blank=True, null=True)),
                ('finish_time', models.DateTimeField(blank=True, null=True)),
                ('created_time', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'db_table': 'django_bgtask',
            },
        ),
    ]
